##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Bound Page Template Tests
"""
import unittest

class Test(unittest.TestCase):

    def testAttributes(self):

        from zope.browserpage.tests.sample import C

        C.index.__func__.foo = 1
        self.assertEqual(C.index.macros, C.index.__func__.macros)
        self.assertEqual(C.index.filename, C.index.__func__.filename)



def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



##############################################################################
#
# Copyright (c) 2001-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Simple View Class Tests
"""
import unittest

class Test_SimpleTestView(unittest.TestCase):

    def _getTargetClass(self):
        from zope.browserpage.tests.simpletestview import SimpleTestView
        return SimpleTestView

    def _makeOne(self, context, request):
        return self._getTargetClass()(context, request)

    def test_simple(self):
        from zope.publisher.browser import TestRequest
        context = DummyContext()
        request = TestRequest()
        view = self._makeOne(context, request)
        self.assertIsNotNone(view['test'])
        out = view()
        self.assertEqual(out.replace('\r\n', '\n'),
                         '<html>\n'
                         '  <body>\n'
                         '    <p>hello world</p>\n'
                         '  </body>\n</html>\n')

class Test_SimpleViewClass(unittest.TestCase):

    def _getTargetClass(self):
        from zope.browserpage.simpleviewclass import SimpleViewClass
        return SimpleViewClass

    def _makeKlass(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test___name__(self):
        klass = self._makeKlass('testsimpleviewclass.pt', name='test.html')
        view = klass(None, None)
        self.assertEqual(view.__name__, 'test.html')

    def test__used_for__(self):
        klass = self._makeKlass('testsimpleviewclass.pt',
                                name='test.html',
                                used_for=self)
        self.assertIs(klass.__used_for__, self)

    def test___getitem___(self):
        klass = self._makeKlass('testsimpleviewclass.pt', name='test.html')
        view = klass(None, None)
        self.assertTrue(view['test'] is not None)
        self.assertRaises(KeyError, view.__getitem__, 'foo')

    def test_w_base_classes(self):
        from zope.publisher.browser import TestRequest
        class BaseClass(object):
            pass

        klass = self._makeKlass('testsimpleviewclass.pt', bases=(BaseClass, ))

        self.assertTrue(issubclass(klass, BaseClass))

        ob = DummyContext()
        request = TestRequest()
        view = klass(ob, request)
        self.assertIsNotNone(view['test'])
        out = view()
        self.assertEqual(out.replace('\r\n', '\n'),
                         '<html>\n'
                         '  <body>\n'
                         '    <p>hello world</p>\n'
                         '  </body>\n</html>\n')

class Test_simple(unittest.TestCase):

    def _getTargetClass(self):
        from zope.browserpage.simpleviewclass import simple
        return simple

    def _makeOne(self, context=None, request=None):
        if context is None:
            context = DummyContext()
        if request is None:
            request = DummyRequest()
        return self._getTargetClass()(context, request)

    def test_class_conforms_to_IBrowserPublisher(self):
        from zope.interface.verify import verifyClass
        from zope.publisher.interfaces.browser import IBrowserPublisher
        verifyClass(IBrowserPublisher, self._getTargetClass())

    def test_browserDefault(self):
        request = DummyRequest()
        view = self._makeOne(request=request)
        self.assertEqual(view.browserDefault(request), (view, ()))

    def test_publishTraverse_not_index_raises_NotFound(self):
        from zope.publisher.interfaces import NotFound
        request = DummyRequest()
        view = self._makeOne(request=request)
        self.assertRaises(NotFound, view.publishTraverse, request, 'nonesuch')

    def test_publishTraverse_w_index_returns_index(self):
        request = DummyRequest()
        view = self._makeOne(request=request)
        index = view.index = DummyTemplate()
        self.assertTrue(view.publishTraverse(request, 'index.html') is index)

    def test___getitem___uses_index_macros(self):
        view = self._makeOne()
        view.index = index = DummyTemplate()
        index.macros = {}
        index.macros['aaa'] = aaa = object()
        self.assertTrue(view['aaa'] is aaa)

    def test___call___no_args_no_kw(self):
        view = self._makeOne()
        view.index = index = DummyTemplate()
        result = view()
        self.assertTrue(result is index)
        self.assertEqual(index._called_with, ((), {}))

    def test___call___w_args_no_kw(self):
        view = self._makeOne()
        view.index = index = DummyTemplate()
        result = view('abc')
        self.assertTrue(result is index)
        self.assertEqual(index._called_with, (('abc',), {}))

    def test___call___no_args_w_kw(self):
        view = self._makeOne()
        view.index = index = DummyTemplate()
        result = view(foo='bar')
        self.assertTrue(result is index)
        self.assertEqual(index._called_with, ((), {'foo': 'bar'}))

    def test___call___w_args_w_kw(self):
        view = self._makeOne()
        view.index = index = DummyTemplate()
        result = view('abc', foo='bar')
        self.assertTrue(result is index)
        self.assertEqual(index._called_with, (('abc',), {'foo': 'bar'}))


class DummyContext:
    pass

class DummyResponse:
    pass

class DummyRequest:
    debug = False
    response = DummyResponse()

class DummyTemplate:
    def __call__(self, *args, **kw):
        self._called_with = (args, kw)
        return self

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(Test_SimpleTestView),
        unittest.makeSuite(Test_SimpleViewClass),
        unittest.makeSuite(Test_simple),
    ))



##############################################################################
#
# Copyright (c) 2017 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import unittest

from zope.interface import Interface

from zope.browserpage.metaconfigure import page
from zope.browserpage.metaconfigure import view
from zope.browserpage.metaconfigure import simple
from zope.browserpage.metaconfigure import _handle_menu
from zope.browserpage.metaconfigure import _handle_allowed_attributes

from zope.configuration.exceptions import ConfigurationError

class Context(object):

    class info(object):
        file = __file__
        line = 1

    def __init__(self):
        self.actions = []

    def path(self, s):
        return s

    def action(self, **kwargs):
        self.actions.append(kwargs)

class _AbstractHandlerTest(unittest.TestCase):

    if not hasattr(unittest.TestCase, 'assertRaisesRegex'):
        # PY2:
        assertRaisesRegex = unittest.TestCase.assertRaisesRegexp  # pragma: no cover

    def setUp(self):
        self.context = Context()

    def _call(self, **kwargs):
        context = self.context
        self._callFUT(context, 'Name', None, **kwargs)
        return context

    def _check_raises(self, regex, **kwargs):
        with self.assertRaisesRegex(ConfigurationError, regex):
            self._call(**kwargs)

class TestPage(_AbstractHandlerTest):

    def _callFUT(self, *args, **kwargs):
        page(*args, **kwargs)

    def test_no_class_no_template(self):
        self._check_raises('Must specify a class or template')

    def test_template_attribute(self):
        self._check_raises(
            'Attribute and template cannot be used together',
            template=self, attribute="attr")

    def test_template_not_exists(self):
        self._check_raises(
            "No such file",
            template=__name__)

    def test_class_missing_attribute(self):
        self._check_raises(
            "The provided class doesn't have the specified attribute",
            class_=type(self), attribute="does not exist")

    def test_class_with_browser_default(self):
        class BrowserDefault(object):
            def browserDefault(self):
                raise AssertionError("Not called")
        context = self._call(
            class_=BrowserDefault)

        action = context.actions[1]
        register = action['args']
        new_class = register[1]
        self.assertEqual(new_class.browserDefault,
                         BrowserDefault.browserDefault)

    def test_class_with_allowed_attributes(self):
        class BrowserDefault(object):
            def foo(self):
                raise AssertionError("foo called")

            def browserDefault(self):
                raise AssertionError("Not called")
        context = self._call(
            class_=BrowserDefault, allowed_attributes=["foo", ])

        action = context.actions[1]
        register = action['args']
        new_class = register[1]
        self.assertEqual(new_class.browserDefault,
                         BrowserDefault.browserDefault)
        self.assertEqual(new_class._simple__whitelist, {"foo"})

    def test_class_with_allowed_interface(self):
        class BrowserDefault(object):
            def foo(self):
                raise AssertionError("foo called")
            def browserDefault(self):
                raise AssertionError("Not called")
        class IFoo(Interface):
            def foo():
                pass
        context = self._call(
            class_=BrowserDefault, allowed_interface=[IFoo])

        action = context.actions[2]
        register = action['args']
        new_class = register[1]
        self.assertEqual(new_class.browserDefault,
                         BrowserDefault.browserDefault)
        self.assertEqual(new_class._simple__whitelist, {"foo"})

    def test_class_implements(self):
        from zope.publisher.interfaces.browser import IBrowserPublisher
        from zope import interface

        class Class(object):
            # How does this arise in the wild?
            # @implementer(IThing) doesn't do it.
            __implements__ = None

        self.assertTrue(hasattr(Class, '__implements__'))
        context = self._call(class_=Class)

        action = context.actions[1]
        register = action['args']
        new_class = register[1]
        self.assertIn(IBrowserPublisher, interface.implementedBy(new_class))

class TestViewPage(_AbstractHandlerTest):

    view = None

    def _callFUT(self, *args, **kwargs):
        self.view = view(self.context, None)
        self.view.page(self.context, 'Name', **kwargs)

    def test_no_attribute(self):
        self._check_raises(
            "Must specify either a template or an attribute name")

class TestView(_AbstractHandlerTest):

    def test_class_without_attribute(self):
        v = view(self.context, None, class_=type(self))
        v.pages.append([None, 'does not exist', None])
        self._call = v

        self._check_raises('Undefined attribute')

    def test_class_without_publish_traverse_name(self):
        class C(object):
            attr = 'the attr'

        v = view(self.context, None, class_=C)
        v.pages.append(['page_name', 'attr', None])

        v()

        action = self.context.actions[2]
        register = action['args']
        new_class = register[1]

        instance = new_class(None, None)
        self.assertEqual(C.attr,
                         instance.publishTraverse(None, 'page_name'))

        with self.assertRaises(LookupError):
            instance.publishTraverse(None, 'something else')

    def test_class_with_publish_traverse_name(self):
        class C(object):
            attr = 'the attr'

            def publishTraverse(self, *args):
                raise AssertionError("Never called")

        v = view(self.context, None, class_=C)
        v.pages.append(['page_name', 'attr', None])

        v()

        action = self.context.actions[2]
        register = action['args']
        new_class = register[1]

        instance = new_class(None, None)
        self.assertEqual(C.attr,
                         instance.publishTraverse(None, 'page_name'))

    def test_class_with_publish_traverse_other_name(self):
        class C(object):
            attr = 'the attr'

            def publishTraverse(self, *args):
                return 'default value'

        v = view(self.context, None, class_=C)
        v.pages.append(['page_name', 'attr', None])

        v()

        action = self.context.actions[2]
        register = action['args']
        new_class = register[1]

        instance = new_class(None, None)
        self.assertEqual('default value',
                         instance.publishTraverse(None, 'something else'))

    def test_template_with_attr_not_equal_name(self):
        import os
        v = view(self.context, None)
        v.page(self.context, 'page_name', attribute='foo',
               template=os.path.join(os.path.dirname(__file__), 'test.pt'))
        v()
        action = self.context.actions[2]
        register = action['args']
        new_class = register[1]

        self.assertTrue(hasattr(new_class, 'foo'))
        self.assertTrue(hasattr(new_class, 'page_name'))

class TestHandleMenu(_AbstractHandlerTest):

    def _call(self, **kwargs):
        args = {
            '_context': self.context,
            'menu': None,
            'title': None,
            'for_': None,
            'name': '',
            'permission': None
        }
        args.update(kwargs)
        return _handle_menu(**args)

    def test_one_or_the_other(self):
        self._check_raises(
            ".*they must both be specified",
            menu='menu', title='')

        self._check_raises(
            ".*they must both be specified",
            menu='', title='menu')

    def test_bad_for(self):
        self._check_raises(
            "Menus can be specified only for single-view",
            menu='menu', title='title',
            for_=(1, 2)
        )

    def test_no_directive(self):
        from zope.browserpage import metaconfigure
        import warnings
        menuItemDirective = metaconfigure.menuItemDirective
        metaconfigure.menuItemDirective = metaconfigure._fallbackMenuItemDirective
        try:
            with warnings.catch_warnings(record=True) as w:
                result = self._call(menu='menu', title='title', for_=(1,))
            self.assertEqual(result, [])
            self.assertEqual(
                'Page directive used with "menu" argument, while "zope.browsermenu" '
                'package is not installed. Doing nothing.',
                str(w[0].message))
        finally:
            metaconfigure.menuItemDirective = menuItemDirective

class TestSimple(unittest.TestCase):

    def test_publish_not_found(self):
        sview = simple(None, None)
        sview.foo = lambda: "bar"
        with self.assertRaises(LookupError):
            sview.publishTraverse(None, "foo")

    def test_publish_whitelist(self):
        sview = simple(None, None)
        sview.foo = lambda: "bar"
        sview._simple__whitelist = {"foo", }
        foo = sview.publishTraverse(None, "foo")
        self.assertEqual(foo(), "bar")

    def test_recursive_call(self):
        sview = simple(None, None)
        sview.__page_attribute__ = '__call__'

        with self.assertRaises(AttributeError):
            sview()



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""View ZPT Tests
"""
import unittest

from zope.component import getGlobalSiteManager
from zope.component.testing import PlacelessSetup
from zope.interface import Interface, implementer

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.browserpage.viewpagetemplatefile import BoundPageTemplate


class I1(Interface):
    pass

@implementer(I1)
class C1(object):
    pass

class InstanceWithContext(object):
    def __init__(self, context):
        self.context = context

class InstanceWithoutContext(object):
    pass


class TestViewZPT(PlacelessSetup, unittest.TestCase):

    def setUp(self):
        super(TestViewZPT, self).setUp()
        self.t = ViewPageTemplateFile('test.pt')
        self.context = C1()

    def testNamespaceContextAvailable(self):
        context = self.context
        request = None

        namespace = self.t.pt_getContext(InstanceWithContext(context), request)
        self.assertTrue(namespace['context'] is context)
        self.assertTrue('views' in namespace)

    def testNamespaceHereNotAvailable(self):
        request = None
        self.assertRaises(AttributeError, self.t.pt_getContext,
                          InstanceWithoutContext(), request)

    def testViewMapper(self):
        the_view = "This is the view"
        the_view_name = "some view name"
        def ViewMaker(*args, **kw):
            return the_view

        from zope.publisher.interfaces import IRequest

        gsm = getGlobalSiteManager()
        gsm.registerAdapter(
            ViewMaker, (I1, IRequest), Interface, the_view_name, event=False)

        @implementer(IRequest)
        class MyRequest(object):
            pass

        request = MyRequest()

        namespace = self.t.pt_getContext(InstanceWithContext(self.context),
                                         request)
        views = namespace['views']
        self.assertTrue(the_view is views[the_view_name])

    def test_debug_flags(self):
        from zope.publisher.browser import TestRequest
        self.request = TestRequest()
        self.request.debug.sourceAnnotations = False
        self.assertFalse('test.pt' in self.t(self))
        self.request.debug.sourceAnnotations = True
        self.assertTrue('test.pt' in self.t(self))

        t = ViewPageTemplateFile('testsimpleviewclass.pt')
        self.request.debug.showTAL = False
        self.assertFalse('metal:' in t(self))
        self.request.debug.showTAL = True
        self.assertTrue('metal:' in t(self))

    def test_render_sets_content_type_unless_set(self):
        from zope.publisher.browser import TestRequest
        t = ViewPageTemplateFile('test.pt')

        self.request = TestRequest()
        response = self.request.response
        self.assertFalse(response.getHeader('Content-Type'))
        t(self)
        self.assertEqual(response.getHeader('Content-Type'), 'text/html')

        self.request = TestRequest()
        response = self.request.response
        response.setHeader('Content-Type', 'application/x-test-junk')
        t(self)
        self.assertEqual(response.getHeader('Content-Type'),
                         'application/x-test-junk')


class TestViewZPTContentType(unittest.TestCase):

    def testInitWithoutType(self):
        t = ViewPageTemplateFile('test.pt')
        t._cook_check()
        self.assertEqual(t.content_type, "text/html")

        t = ViewPageTemplateFile('testxml.pt')
        t._cook_check()
        self.assertEqual(t.content_type, "text/xml")

    def testInitWithType(self):
        t = ViewPageTemplateFile('test.pt', content_type="text/plain")
        t._cook_check()
        self.assertEqual(t.content_type, "text/plain")

        t = ViewPageTemplateFile('testxml.pt', content_type="text/plain")
        t._cook_check()
        self.assertEqual(t.content_type, "text/xml")


class TestBoundPageTemplate(unittest.TestCase):

    def test_call_self_none(self):
        args = []
        def func(*a):
            args.extend(a)

        bpt = BoundPageTemplate(func, None)
        bpt(self, 1, 2)

        self.assertEqual(args, [self, 1, 2])

    def test_cannot_set_attr(self):
        bpt = BoundPageTemplate(None, None)
        with self.assertRaises(AttributeError):
            bpt.thing = 1

        with self.assertRaises(AttributeError):
            bpt.__func__ = 1

        with self.assertRaises(AttributeError):
            bpt.__self__ = 1

    def test_repr(self):
        ob = "WackyWavingInflatableArmFlailingTubeMan"

        bpt = BoundPageTemplate(None, ob)
        self.assertEqual(repr(bpt),
                         "<BoundPageTemplateFile of 'WackyWavingInflatableArmFlailingTubeMan'>")


class TestNoTraverser(unittest.TestCase):

    def test_none(self):
        from zope.browserpage.viewpagetemplatefile import NoTraverser
        self.assertIsNone(NoTraverser(self, self))

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser page ZCML configuration code
"""
import os

from zope.component import queryMultiAdapter
from zope.component.interface import provideInterface
from zope.component.zcml import handler
from zope.interface import implementer, classImplements, Interface
from zope.publisher.interfaces import NotFound
from zope.security.checker import CheckerPublic, Checker, defineChecker
from zope.configuration.exceptions import ConfigurationError
from zope.pagetemplate.engine import Engine
from zope.pagetemplate.engine import _Engine
from zope.pagetemplate.engine import TrustedEngine
from zope.pagetemplate.engine import _TrustedEngine
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.browser import BrowserView

from zope.browserpage.simpleviewclass import SimpleViewClass
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile

def _fallbackMenuItemDirective(_context, *args, **kwargs):
    import warnings
    warnings.warn_explicit(
        'Page directive used with "menu" argument, while "zope.browsermenu" '
        'package is not installed. Doing nothing.',
        UserWarning,
        _context.info.file, _context.info.line)
    return []

try:
    from zope.browsermenu.metaconfigure import menuItemDirective
except ImportError: # pragma: no cover
    menuItemDirective = _fallbackMenuItemDirective

# There are three cases we want to suport:
#
# Named view without pages (single-page view)
#
#     <browser:page
#         for=".IContact.IContactInfo."
#         name="info.html"
#         template="info.pt"
#         class=".ContactInfoView."
#         permission="zope.View"
#         />
#
# Unnamed view with pages (multi-page view)
#
#     <browser:pages
#         for=".IContact."
#         class=".ContactEditView."
#         permission="ZopeProducts.Contact.ManageContacts"
#         >
#
#       <browser:page name="edit.html"       template="edit.pt" />
#       <browser:page name="editAction.html" attribute="action" />
#       </browser:pages>
#
# Named view with pages (add view is a special case of this)
#
#        <browser:view
#            name="ZopeProducts.Contact"
#            for="Zope.App.OFS.Container.IAdding."
#            class=".ContactAddView."
#            permission="ZopeProducts.Contact.ManageContacts"
#            >
#
#          <browser:page name="add.html"    template="add.pt" />
#          <browser:page name="action.html" attribute="action" />
#          </browser:view>

# We'll also provide a convenience directive for add views:
#
#        <browser:add
#            name="ZopeProducts.Contact"
#            class=".ContactAddView."
#            permission="ZopeProducts.Contact.ManageContacts"
#            >
#
#          <browser:page name="add.html"    template="add.pt" />
#          <browser:page name="action.html" attribute="action" />
#          </browser:view>

# page

def _norm_template(_context, template):
    template = os.path.abspath(str(_context.path(template)))
    if not os.path.isfile(template):
        raise ConfigurationError("No such file", template)
    return template


def page(_context, name, permission, for_=Interface,
         layer=IDefaultBrowserLayer, template=None, class_=None,
         allowed_interface=None, allowed_attributes=None,
         attribute='__call__', menu=None, title=None):
    _handle_menu(_context, menu, title, [for_], name, permission, layer)
    required = {}

    permission = _handle_permission(_context, permission)

    if not (class_ or template):
        raise ConfigurationError("Must specify a class or template")

    if attribute != '__call__':
        if template:
            raise ConfigurationError(
                "Attribute and template cannot be used together.")

        assert class_, "Must have class if attribute is used"

    if template:
        template = _norm_template(_context, template)
        required['__getitem__'] = permission

    # TODO: new __name__ attribute must be tested
    if class_:
        if attribute != '__call__':
            if not hasattr(class_, attribute):
                raise ConfigurationError(
                    "The provided class doesn't have the specified attribute "
                    )
        if template:
            # class and template
            new_class = SimpleViewClass(template, bases=(class_, ), name=name)
        else:
            cdict = {}
            cdict['__name__'] = name
            cdict['__page_attribute__'] = attribute
            new_class = type(class_.__name__, (class_, simple,), cdict)

        if hasattr(class_, '__implements__'):
            classImplements(new_class, IBrowserPublisher)

    else:
        # template
        new_class = SimpleViewClass(template, name=name)

    for n in (attribute, 'browserDefault', '__call__', 'publishTraverse'):
        required[n] = permission

    _handle_allowed_interface(_context, allowed_interface, permission,
                              required)
    _handle_allowed_attributes(_context, allowed_attributes, permission,
                               required)

    _handle_for(_context, for_)
    new_class._simple__whitelist = set(required) - set([attribute, 'browserDefault', '__call__', 'publishTraverse'])

    defineChecker(new_class, Checker(required))

    _context.action(
        discriminator=('view', (for_, layer), name, IBrowserRequest),
        callable=handler,
        args=('registerAdapter',
              new_class, (for_, layer), Interface, name, _context.info),
    )


# pages, which are just a short-hand for multiple page directives.

# Note that a class might want to access one of the defined
# templates. If it does though, it should use getMultiAdapter.

class pages(object):

    def __init__(self, _context, permission, for_=Interface,
                 layer=IDefaultBrowserLayer, class_=None,
                 allowed_interface=None, allowed_attributes=None):
        self.opts = dict(
            for_=for_, permission=permission,
            layer=layer, class_=class_,
            allowed_interface=allowed_interface,
            allowed_attributes=allowed_attributes,
        )

    def page(self, _context, name, attribute='__call__', template=None,
             menu=None, title=None):
        return page(_context,
                    name=name,
                    attribute=attribute,
                    template=template,
                    menu=menu, title=title,
                    **(self.opts))

    def __call__(self):
        return ()

# view (named view with pages)

# This is a different case. We actually build a class with attributes
# for all of the given pages.

class view(object):

    default = None

    def __init__(self, _context, permission, for_=Interface,
                 name='', layer=IDefaultBrowserLayer, class_=None,
                 allowed_interface=None, allowed_attributes=None,
                 menu=None, title=None, provides=Interface):

        _handle_menu(_context, menu, title, [for_], name, permission, layer)

        permission = _handle_permission(_context, permission)

        self.args = (_context, name, (for_, layer), permission, class_,
                     allowed_interface, allowed_attributes)

        self.pages = []
        self.menu = menu
        self.provides = provides

    def page(self, _context, name, attribute=None, template=None):
        if template:
            template = _norm_template(_context, template)
        else:
            if not attribute:
                raise ConfigurationError(
                    "Must specify either a template or an attribute name")

        self.pages.append((name, attribute, template))
        return ()

    def defaultPage(self, _context, name):
        self.default = name
        return ()

    def __call__(self):
        (_context, name, (for_, layer), permission, class_,
         allowed_interface, allowed_attributes) = self.args

        required = {}

        cdict = {}
        pages = {}

        for pname, attribute, template in self.pages:

            if template:
                cdict[pname] = ViewPageTemplateFile(template)
                if attribute and attribute != name:
                    cdict[attribute] = cdict[pname]
            else:
                if not hasattr(class_, attribute):
                    raise ConfigurationError("Undefined attribute",
                                             attribute)

            attribute = attribute or pname
            required[pname] = permission

            pages[pname] = attribute

        # This should go away, but noone seems to remember what to do. :-(
        if hasattr(class_, 'publishTraverse'):

            def publishTraverse(self, request, name,
                                pages=pages, getattr=getattr):

                if name in pages:
                    return getattr(self, pages[name])
                view = queryMultiAdapter((self, request), name=name)
                if view is not None:
                    return view

                m = class_.publishTraverse.__get__(self)
                return m(request, name)

        else:
            def publishTraverse(self, request, name,
                                pages=pages, getattr=getattr):

                if name in pages:
                    return getattr(self, pages[name])
                view = queryMultiAdapter((self, request), name=name)
                if view is not None:
                    return view

                raise NotFound(self, name, request)

        cdict['publishTraverse'] = publishTraverse

        if not hasattr(class_, 'browserDefault'):
            if self.default or self.pages:
                _default = self.default or self.pages[0][0]
                cdict['browserDefault'] = (
                    lambda self, request, default=_default:
                    (self, (default, ))
                )
            elif providesCallable(class_):
                cdict['browserDefault'] = (
                    lambda self, request: (self, ())
                )

        bases = (simple,) if class_ is None else (class_, simple)

        try:
            cname = str(name)
        except Exception: # pragma: no cover
            cname = "GeneratedClass"

        cdict['__name__'] = name
        newclass = type(cname, bases, cdict)

        for n in ('publishTraverse', 'browserDefault', '__call__'):
            required[n] = permission

        _handle_allowed_interface(_context, allowed_interface, permission,
                                  required)
        _handle_allowed_attributes(_context, allowed_attributes, permission,
                                   required)
        _handle_for(_context, for_)

        defineChecker(newclass, Checker(required))

        if self.provides is not None:
            _context.action(
                discriminator=None,
                callable=provideInterface,
                args=('', self.provides)
            )

        _context.action(
            discriminator=('view', (for_, layer), name, self.provides),
            callable=handler,
            args=('registerAdapter',
                  newclass, (for_, layer), self.provides, name,
                  _context.info),
        )

def _handle_menu(_context, menu, title, for_, name, permission,
                 layer=IDefaultBrowserLayer):
    if not menu and not title:
        # Neither of them
        return []

    if not menu or not title:
        # Only one or the other
        raise ConfigurationError(
            "If either menu or title are specified, they must "
            "both be specified.")

    if len(for_) != 1:
        raise ConfigurationError(
            "Menus can be specified only for single-view, not for "
            "multi-views.")

    return menuItemDirective(
        _context, menu, for_[0], '@@' + name, title,
        permission=permission, layer=layer)


def _handle_permission(_context, permission):
    if permission == 'zope.Public':
        permission = CheckerPublic

    return permission

def _handle_allowed_interface(_context, allowed_interface, permission,
                              required):
    # Allow access for all names defined by named interfaces
    if allowed_interface:
        for i in allowed_interface:
            _context.action(
                discriminator=None,
                callable=provideInterface,
                args=(None, i)
            )

            for name in i:
                required[name] = permission

def _handle_allowed_attributes(_context, allowed_attributes, permission,
                               required):
    # Allow access for all named attributes
    if allowed_attributes:
        for name in allowed_attributes:
            required[name] = permission

def _handle_for(_context, for_):
    if for_ is not None:
        _context.action(
            discriminator=None,
            callable=provideInterface,
            args=('', for_)
        )

@implementer(IBrowserPublisher)
class simple(BrowserView):

    __page_attribute__ = '__call__'

    def publishTraverse(self, request, name):
        if name in getattr(self, "_simple__whitelist", []):
            self.__page_attribute__ = name
            return self
        else:
            raise NotFound(self, name, request)

    def __call__(self, *a, **k):
        # If a class doesn't provide it's own call, then get the attribute
        attr = self.__page_attribute__
        if attr == '__call__':
            raise AttributeError("__call__")

        meth = getattr(self, attr)
        return meth(*a, **k)

    def browserDefault(self, request):
        # If a class doesn't provide it's own browserDefault, then get the attribute
        # given by the __page_attribute__.

        attr = self.__page_attribute__
        if attr == 'browserDefault':
            # safety guard against recursion error:
            raise AttributeError("browserDefault")  # pragma: no cover

        meth = getattr(self, attr)
        return (meth, "")

def providesCallable(class_):
    if hasattr(class_, '__call__'):
        for c in class_.__mro__:
            if '__call__' in c.__dict__:
                return True
    return False


def expressiontype(_context, name, handler):
    _context.action(
        discriminator=("tales:expressiontype", name),
        callable=registerType,
        args=(name, handler)
    )

def registerType(name, handler):
    Engine.registerType(name, handler)
    TrustedEngine.registerType(name, handler)


def clear():
    Engine.__init__()
    _Engine(Engine)
    TrustedEngine.__init__()
    _TrustedEngine(TrustedEngine)


try:
    from zope.testing.cleanup import addCleanUp
except ImportError: # pragma: no cover
    pass
else:
    addCleanUp(clear)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for browser:page directive and friends
"""

import os
import unittest
from io import StringIO

from zope import component
from zope.interface import Interface, implementer, directlyProvides

import zope.security.management
from zope.configuration.xmlconfig import xmlconfig, XMLConfig
from zope.configuration.exceptions import ConfigurationError
from zope.publisher.browser import TestRequest

from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces.browser import IBrowserRequest

from zope.security.proxy import removeSecurityProxy, ProxyFactory
from zope.security.permission import Permission
from zope.security.interfaces import IPermission
from zope.testing import cleanup
from zope.traversing.adapters import DefaultTraversable
from zope.traversing.interfaces import ITraversable

import zope.publisher.defaultview
import zope.browserpage
import zope.browsermenu
from zope.browsermenu.menu import getFirstMenuItem
from zope.browsermenu.interfaces import IMenuItemType
#from zope.component.testfiles.views import IC, V1, VZMI, R1, IV

tests_path = os.path.dirname(__file__)

template = u"""<configure
   xmlns='http://namespaces.zope.org/zope'
   xmlns:browser='http://namespaces.zope.org/browser'
   i18n_domain='zope'>
   %s
   </configure>"""

class templateclass(object):
    def data(self):
        return 42

class IR(Interface):
    pass

class IV(Interface):
    def index():
        "A method"

class IC(Interface):
    pass

@implementer(IV)
class V1(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def index(self):
        return 'V1 here'

    def action(self):
        return 'done'

class VZMI(V1):
    def index(self):
        raise AssertionError("Not called")

@implementer(IV)
class R1(object):
    pass

class RZMI(R1):
    pass

class V2(V1, object):

    def action(self):
        return self.action2()

    def action2(self):
        return "done"

class VT(V1, object):
    def publishTraverse(self, request, name):
        raise AssertionError("Not called")

@implementer(IC)
class Ob(object):
    pass

ob = Ob()

class NCV(object):
    "non callable view"

    def __init__(self, context, request):
        pass

class CV(NCV):
    "callable view"
    def __call__(self):
        raise AssertionError("Not called")


@implementer(Interface)
class C_w_implements(NCV):

    def index(self):
        return self

class ITestLayer(IBrowserRequest):
    """Test Layer."""

class ITestSkin(ITestLayer):
    """Test Skin."""


class ITestMenu(Interface):
    """Test menu."""

directlyProvides(ITestMenu, IMenuItemType)

class Test(cleanup.CleanUp, unittest.TestCase):

    def setUp(self):
        super(Test, self).setUp()
        XMLConfig('meta.zcml', zope.browserpage)()
        XMLConfig('meta.zcml', zope.browsermenu)()
        component.provideAdapter(DefaultTraversable, (None,), ITraversable, )
        zope.security.management.newInteraction()
        self.request = TestRequest()

    def testPage(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(template % (
            '''
            <browser:page
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                attribute="index"
                />
            '''
            )))

        v = component.queryMultiAdapter((ob, self.request), name='test')
        self.assertTrue(issubclass(v.__class__, V1))


    def testPageWithClassWithMenu(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')


        xmlconfig(StringIO(template % (
            '''
            <browser:menu
                id="test_menu" title="Test menu" />
            <browser:page
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                template="%s"
                menu="test_menu"
                title="Test View"
                />
            ''' % testtemplate
            )))
        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
        self.assertEqual(menuItem["title"], "Test View")
        self.assertEqual(menuItem["action"], "@@test")
        v = component.queryMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v().replace('\r\n', '\n'),
                         "<html><body><p>test</p></body></html>\n")


    def testPageWithTemplateWithMenu(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')

        xmlconfig(StringIO(template % (
            '''
            <browser:menu
                id="test_menu" title="Test menu"/>
            <browser:page
                name="test"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                template="%s"
                menu="test_menu"
                title="Test View"
                />
            ''' % testtemplate
            )))

        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
        self.assertEqual(menuItem["title"], "Test View")
        self.assertEqual(menuItem["action"], "@@test")
        v = component.queryMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v().replace('\r\n', '\n'),
                         "<html><body><p>test</p></body></html>\n")


    def testPageInPagesWithTemplateWithMenu(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')

        xmlconfig(StringIO(template % (
            '''
            <browser:menu
                id="test_menu" title="Test menu" />
            <browser:pages
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public">
              <browser:page
                  name="test"
                  template="%s"
                  menu="test_menu"
                  title="Test View"
                  />
            </browser:pages>
            ''' % testtemplate
            )))

        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
        self.assertEqual(menuItem["title"], "Test View")
        self.assertEqual(menuItem["action"], "@@test")
        v = component.queryMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v().replace('\r\n', '\n'),
                         "<html><body><p>test</p></body></html>\n")


    def testPageInPagesWithClassWithMenu(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')

        xmlconfig(StringIO(template % (
            '''
            <browser:menu
                id="test_menu" title="Test menu" />
            <browser:pages
                for="zope.browserpage.tests.test_page.IC"
                class="zope.browserpage.tests.test_page.V1"
                permission="zope.Public">
              <browser:page
                  name="test"
                  template="%s"
                  menu="test_menu"
                  title="Test View"
                  />
            </browser:pages>
            ''' % testtemplate
            )))

        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
        self.assertEqual(menuItem["title"], "Test View")
        self.assertEqual(menuItem["action"], "@@test")
        v = component.queryMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v().replace('\r\n', '\n'),
                         "<html><body><p>test</p></body></html>\n")

    def testSkinPage(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.VZMI"
                for="zope.browserpage.tests.test_page.IC"
                layer="zope.browserpage.tests.test_page.ITestLayer"
                permission="zope.Public"
                attribute="index"
                />
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                attribute="index"
                />
            '''
        ))

        v = component.queryMultiAdapter((ob, self.request), name='test')
        self.assertTrue(issubclass(v.__class__, V1))
        v = component.queryMultiAdapter(
            (ob, TestRequest(skin=ITestSkin)), name='test')
        self.assertTrue(issubclass(v.__class__, VZMI))


    def testInterfaceProtectedPage(self):
        xmlconfig(StringIO(
            template %
            u'''
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.V1"
                attribute="index"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                allowed_interface="zope.browserpage.tests.test_page.IV"
                />
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='test')
        v = ProxyFactory(v)
        self.assertEqual(v.index(), 'V1 here')
        self.assertRaises(Exception, getattr, v, 'action')

    def testAttributeProtectedPage(self):
        xmlconfig(StringIO(
            template %
            u'''
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.V2"
                for="zope.browserpage.tests.test_page.IC"
                attribute="action"
                permission="zope.Public"
                allowed_attributes="action2"
                />
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='test')
        v = ProxyFactory(v)
        self.assertEqual(v.action(), 'done')
        self.assertEqual(v.action2(), 'done')
        self.assertRaises(Exception, getattr, v, 'index')

    def testAttributeProtectedView(self):
        xmlconfig(StringIO(
            template %
            u'''
            <browser:view name="test"
                class="zope.browserpage.tests.test_page.V2"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                allowed_attributes="action2"
                >
              <browser:page name="index.html" attribute="action" />
           </browser:view>
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='test')
        v = ProxyFactory(v)
        page = v.publishTraverse(self.request, 'index.html')
        self.assertEqual(page(), 'done')
        self.assertEqual(v.action2(), 'done')
        self.assertRaises(Exception, getattr, page, 'index')

    def testInterfaceAndAttributeProtectedPage(self):
        xmlconfig(StringIO(
            template %
            u'''
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                attribute="index"
                allowed_attributes="action"
                allowed_interface="zope.browserpage.tests.test_page.IV"
                />
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v.index(), 'V1 here')
        self.assertEqual(v.action(), 'done')

    def testDuplicatedInterfaceAndAttributeProtectedPage(self):
        xmlconfig(StringIO(
            template %
            u'''
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                attribute="index"
                permission="zope.Public"
                allowed_attributes="action index"
                allowed_interface="zope.browserpage.tests.test_page.IV"
                />
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v.index(), 'V1 here')
        self.assertEqual(v.action(), 'done')

    def test_class_w_implements(self):
        xmlconfig(StringIO(
            template %
            u'''
            <browser:page
                name="test"
                class="
             zope.browserpage.tests.test_page.C_w_implements"
                for="zope.browserpage.tests.test_page.IC"
                attribute="index"
                permission="zope.Public"
                />
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='test')
        self.assertEqual(v.index(), v)
        self.assertTrue(IBrowserPublisher.providedBy(v))

    def testIncompleteProtectedPageNoPermission(self):
        self.assertRaises(
            ConfigurationError,
            xmlconfig,
            StringIO(
                template %
                u'''
            <browser:page name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                attribute="index"
                allowed_attributes="action index"
                />
            '''
            ))


    def testPageViews(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        test3 = os.path.join(tests_path, 'testfiles', 'test3.pt')

        xmlconfig(StringIO(
            template %
            u'''
            <browser:pages
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                >

              <browser:page name="index.html" attribute="index" />
              <browser:page name="action.html" attribute="action" />
              <browser:page name="test.html" template="%s" />
            </browser:pages>
            ''' % test3
        ))

        v = component.getMultiAdapter((ob, self.request), name='index.html')
        self.assertEqual(v(), 'V1 here')
        v = component.getMultiAdapter((ob, self.request), name='action.html')
        self.assertEqual(v(), 'done')
        v = component.getMultiAdapter((ob, self.request), name='test.html')
        self.assertEqual(str(v()).replace('\r\n', '\n'),
                         "<html><body><p>done</p></body></html>\n")

    def testNamedViewPageViewsCustomTraversr(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                >

              <browser:page name="index.html" attribute="index" />
              <browser:page name="action.html" attribute="action" />
            </browser:view>
            '''
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        self.assertEqual(view.browserDefault(self.request)[1], (u'index.html', ))


        v = view.publishTraverse(self.request, 'index.html')
        v = removeSecurityProxy(v)
        self.assertEqual(v(), 'V1 here')
        v = view.publishTraverse(self.request, 'action.html')
        v = removeSecurityProxy(v)
        self.assertEqual(v(), 'done')


    def testNamedViewNoPagesForCallable(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.CV"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                />
            '''
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        self.assertEqual(view.browserDefault(self.request), (view, ()))

    def testNamedViewNoPagesForNonCallable(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.NCV"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                />
            '''
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        browser_default = view.browserDefault(self.request)
        self.assertEqual(browser_default[1], '')
        with self.assertRaises(AttributeError) as ctx:
            browser_default[0]()
        self.assertEqual(str(ctx.exception), "__call__")

    def testNamedViewPageViewsNoDefault(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        test3 = os.path.join(tests_path, 'testfiles', 'test3.pt')

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                >

              <browser:page name="index.html" attribute="index" />
              <browser:page name="action.html" attribute="action" />
              <browser:page name="test.html" template="%s" />
            </browser:view>
            ''' % test3
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        self.assertEqual(view.browserDefault(self.request)[1], (u'index.html', ))


        v = view.publishTraverse(self.request, 'index.html')
        v = removeSecurityProxy(v)
        self.assertEqual(v(), 'V1 here')
        v = view.publishTraverse(self.request, 'action.html')
        v = removeSecurityProxy(v)
        self.assertEqual(v(), 'done')
        v = view.publishTraverse(self.request, 'test.html')
        v = removeSecurityProxy(v)
        self.assertEqual(str(v()).replace('\r\n', '\n'),
                         "<html><body><p>done</p></body></html>\n")

    def testNamedViewPageViewsWithDefault(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)
        test3 = os.path.join(tests_path, u'testfiles', u'test3.pt')

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                >

              <browser:defaultPage name="test.html" />
              <browser:page name="index.html" attribute="index" />
              <browser:page name="action.html" attribute="action" />
              <browser:page name="test.html" template="%s" />
            </browser:view>
            ''' % test3
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        self.assertEqual(view.browserDefault(self.request)[1], (u'test.html', ))


        v = view.publishTraverse(self.request, 'index.html')
        v = removeSecurityProxy(v)
        self.assertEqual(v(), 'V1 here')
        v = view.publishTraverse(self.request, 'action.html')
        v = removeSecurityProxy(v)
        self.assertEqual(v(), 'done')
        v = view.publishTraverse(self.request, 'test.html')
        v = removeSecurityProxy(v)
        self.assertEqual(str(v()).replace('\r\n', '\n'),
                         "<html><body><p>done</p></body></html>\n")

    def testTraversalOfPageForView(self):
        """Tests proper traversal of a page defined for a view."""

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public" />

            <browser:page name="index.html"
                for="zope.browserpage.tests.test_page.IV"
                class="zope.browserpage.tests.test_page.CV"
                permission="zope.Public" />
            '''
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        view.publishTraverse(self.request, 'index.html')

    def testTraversalOfPageForViewWithPublishTraverse(self):
        """Tests proper traversal of a page defined for a view.

        This test is different from testTraversalOfPageForView in that it
        tests the behavior on a view that has a publishTraverse method --
        the implementation of the lookup is slightly different in such a
        case.
        """
        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.VT"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public" />

            <browser:page name="index.html"
                for="zope.browserpage.tests.test_page.IV"
                class="zope.browserpage.tests.test_page.CV"
                permission="zope.Public" />
            '''
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        view = removeSecurityProxy(view)
        view.publishTraverse(self.request, 'index.html')

    def testProtectedPageViews(self):
        component.provideUtility(Permission('p', 'P'), IPermission, 'p')

        request = TestRequest()
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <include package="zope.security" file="meta.zcml" />

            <permission id="zope.TestPermission" title="Test permission" />

            <browser:pages
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.TestPermission"
                >

              <browser:page name="index.html" attribute="index" />
              <browser:page name="action.html" attribute="action" />
            </browser:pages>
            '''
        ))

        v = component.getMultiAdapter((ob, request), name='index.html')
        v = ProxyFactory(v)
        zope.security.management.getInteraction().add(request)
        self.assertRaises(Exception, v)
        v = component.getMultiAdapter((ob, request), name='action.html')
        v = ProxyFactory(v)
        self.assertRaises(Exception, v)

    def testProtectedNamedViewPageViews(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <include package="zope.security" file="meta.zcml" />

            <permission id="zope.TestPermission" title="Test permission" />

            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                >

              <browser:page name="index.html" attribute="index" />
              <browser:page name="action.html" attribute="action" />
            </browser:view>
            '''
        ))

        view = component.getMultiAdapter((ob, self.request), name='test')
        self.assertEqual(view.browserDefault(self.request)[1], (u'index.html', ))

        v = view.publishTraverse(self.request, 'index.html')
        self.assertEqual(v(), 'V1 here')

    def testSkinnedPageView(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:pages
                class="zope.browserpage.tests.test_page.V1"
                for="*"
                permission="zope.Public"
                >
              <browser:page name="index.html" attribute="index" />
            </browser:pages>

            <browser:pages
                class="zope.browserpage.tests.test_page.V1"
                for="*"
                layer="zope.browserpage.tests.test_page.ITestLayer"
                permission="zope.Public"
                >
              <browser:page name="index.html" attribute="action" />
            </browser:pages>
            '''
        ))

        v = component.getMultiAdapter((ob, self.request), name='index.html')
        self.assertEqual(v(), 'V1 here')
        v = component.getMultiAdapter((ob, TestRequest(skin=ITestSkin)),
                                      name='index.html')
        self.assertEqual(v(), 'done')


    def test_template_page(self):
        path = os.path.join(tests_path, 'testfiles', 'test.pt')

        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='index.html'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:page
                name="index.html"
                template="%s"
                permission="zope.Public"
                for="zope.browserpage.tests.test_page.IC" />
            ''' % path
        ))

        v = component.getMultiAdapter((ob, self.request), name='index.html')
        self.assertEqual(v().strip(), '<html><body><p>test</p></body></html>')

    def test_page_menu_within_different_layers(self):
        path = os.path.join(tests_path, 'testfiles', 'test.pt')
        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='index.html'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:menu
                id="test_menu"
                title="Test menu"
                interface="zope.browserpage.tests.test_page.ITestMenu"/>

            <browser:page
                name="index.html"
                permission="zope.Public"
                for="zope.browserpage.tests.test_page.IC"
                template="%s"
                menu="test_menu" title="Index"/>

            <browser:page
                name="index.html"
                permission="zope.Public"
                for="zope.browserpage.tests.test_page.IC"
                menu="test_menu" title="Index"
                template="%s"
                layer="zope.browserpage.tests.test_page.ITestLayer"/>
            ''' % (path, path)
            ))

        v = component.getMultiAdapter((ob, self.request), name='index.html')
        self.assertEqual(v().strip(), '<html><body><p>test</p></body></html>')

    def testtemplateWClass(self):
        path = os.path.join(tests_path, 'testfiles', 'test2.pt')

        self.assertEqual(
            component.queryMultiAdapter((ob, self.request), name='index.html'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:page
                name="index.html"
                template="%s"
                permission="zope.Public"
          class="zope.browserpage.tests.test_page.templateclass"
                for="zope.browserpage.tests.test_page.IC" />
            ''' % path
        ))

        v = component.getMultiAdapter((ob, self.request), name='index.html')
        self.assertEqual(v().strip(), '<html><body><p>42</p></body></html>')

    def testProtectedtemplate(self):

        path = os.path.join(tests_path, 'testfiles', 'test.pt')

        request = TestRequest()
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='test'),
            None)

        xmlconfig(StringIO(
            template %
            u'''
            <include package="zope.security" file="meta.zcml" />

            <permission id="zope.TestPermission" title="Test permission" />

            <browser:page
                name="xxx.html"
                template="%s"
                permission="zope.TestPermission"
                for="zope.browserpage.tests.test_page.IC" />
            ''' % path
        ))

        xmlconfig(StringIO(
            template %
            u'''
            <browser:page
                name="index.html"
                template="%s"
                permission="zope.Public"
                for="zope.browserpage.tests.test_page.IC" />
            ''' % path
        ))

        v = component.getMultiAdapter((ob, request), name='xxx.html')
        v = ProxyFactory(v)
        zope.security.management.getInteraction().add(request)
        self.assertRaises(Exception, v)

        v = component.getMultiAdapter((ob, request), name='index.html')
        v = ProxyFactory(v)
        self.assertEqual(v().strip(), '<html><body><p>test</p></body></html>')


    def testtemplateNoName(self):
        path = os.path.join(tests_path, 'testfiles', 'test.pt')
        self.assertRaises(
            ConfigurationError,
            xmlconfig,
            StringIO(
                template %
                u'''
            <browser:page
                template="%s"
                for="zope.browserpage.tests.test_page.IC"
                />
            ''' % path
            ))

    def testtemplateAndPage(self):
        path = os.path.join(tests_path, u'testfiles', u'test.pt')
        self.assertRaises(
            ConfigurationError,
            xmlconfig,
            StringIO(
                template %
                u'''
            <browser:view
                name="index.html"
                template="%s"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                >
              <browser:page name="foo.html" attribute="index" />
            </browser:view>
            ''' % path
            ))

    def testViewThatProvidesAnInterface(self):
        request = TestRequest()
        self.assertEqual(
            component.queryMultiAdapter((ob, request), IV, name='test'), None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                />
            '''
        ))

        v = component.queryMultiAdapter((ob, request), IV, name='test')
        self.assertEqual(v, None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                name="test"
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                provides="zope.browserpage.tests.test_page.IV"
                permission="zope.Public"
                />
            '''
        ))

        v = component.queryMultiAdapter((ob, request), IV, name='test')
        self.assertTrue(isinstance(v, V1))

    def testUnnamedViewThatProvidesAnInterface(self):
        request = TestRequest()
        self.assertEqual(component.queryMultiAdapter((ob, request), IV),
                         None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                permission="zope.Public"
                />
            '''
        ))

        v = component.queryMultiAdapter((ob, request), IV)
        self.assertEqual(v, None)

        xmlconfig(StringIO(
            template %
            u'''
            <browser:view
                class="zope.browserpage.tests.test_page.V1"
                for="zope.browserpage.tests.test_page.IC"
                provides="zope.browserpage.tests.test_page.IV"
                permission="zope.Public"
                />
            '''
        ))

        v = component.queryMultiAdapter((ob, request), IV)

        self.assertTrue(isinstance(v, V1))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests to check talesapi zcml configuration
"""
import unittest
from io import StringIO

from zope.configuration.xmlconfig import xmlconfig, XMLConfig
from zope.pagetemplate.engine import Engine
import zope.browserpage

from zope.component.testing import PlacelessSetup

template = u"""<configure
   xmlns='http://namespaces.zope.org/zope'
   xmlns:tales='http://namespaces.zope.org/tales'>
   %s
   </configure>"""


class Handler(object):
    pass

class Test(PlacelessSetup, unittest.TestCase):

    def setUp(self):
        super(Test, self).setUp()
        XMLConfig('meta.zcml', zope.browserpage)()

    def testExpressionType(self):
        xmlconfig(StringIO(
            template %
            u"""
            <tales:expressiontype
              name="test"
              handler="zope.browserpage.tests.test_expressiontype.Handler"
              />
            """
        ))
        self.assertIn("test", Engine.getTypes())
        self.assertIs(Handler, Engine.getTypes()['test'])

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2005-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
"""
import os
import os.path
import unittest

from zope.browserpage import namedtemplate

import zope.component.testing
import zope.traversing.adapters

class TestImplementation(unittest.TestCase):

    def test_call(self):
        inst = namedtemplate.implementation(None)


        nt = namedtemplate.NamedTemplate('name')
        nti = inst(nt)

        self.assertEqual(nti.descriptor, nt)

class TestNamedTemplateImplementation(unittest.TestCase):

    def test_bad_descriptor(self):
        with self.assertRaises(TypeError):
            namedtemplate.NamedTemplateImplementation(None)


class TestNamedTemplate(unittest.TestCase):

    def test_no_instance(self):
        class X(object):
            nt = namedtemplate.NamedTemplate('name')

        self.assertIsInstance(X.nt, namedtemplate.NamedTemplate)

    def test_call_no_adapter(self):
        class X(object):
            nt = namedtemplate.NamedTemplate('name')

        with self.assertRaises(LookupError):
            getattr(X(), 'nt')

        with self.assertRaises(LookupError):
            X.nt(X())

class TestNamedTemplatePathAdapter(unittest.TestCase):

    def test_getitem_no_adapter(self):

        adapter = namedtemplate.NamedTemplatePathAdapter(self)

        with self.assertRaises(LookupError):
            _ = adapter['name']

def pageSetUp(test):
    zope.component.testing.setUp(test)
    zope.component.provideAdapter(
        zope.traversing.adapters.DefaultTraversable,
        [None],
    )


def test_suite():
    import doctest
    filename = os.path.join(os.pardir, 'namedtemplate.rst')
    return unittest.TestSuite([
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocFileSuite(
            filename,
            setUp=pageSetUp, tearDown=zope.component.testing.tearDown,
            globs={'__file__':  os.path.abspath(os.path.join(os.path.dirname(__file__), filename))}
        )
    ])



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Simple View Class
"""
__docformat__ = 'restructuredtext'

import sys
from zope.interface import implementer
from zope.publisher.browser import BrowserView
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces import NotFound
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile

@implementer(IBrowserPublisher)
class simple(BrowserView):

    def browserDefault(self, request):
        return self, ()

    def publishTraverse(self, request, name):
        if name == 'index.html':
            return self.index

        raise NotFound(self, name, request)

    def __getitem__(self, name):
        return self.index.macros[name]

    def __call__(self, *args, **kw):
        return self.index(*args, **kw)


def SimpleViewClass(src, offering=None, used_for=None, bases=(), name=u''):
    if offering is None:
        offering = sys._getframe(1).f_globals

    bases += (simple, )

    class_ = type("SimpleViewClass from %s" % src, bases,
                  {'index': ViewPageTemplateFile(src, offering),
                   '__name__': name})

    if used_for is not None:
        class_.__used_for__ = used_for

    return class_



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""zope.browserpage setup
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


long_description = (read('README.rst') + '\n\n' + read('CHANGES.rst'))


def alltests():
    import sys
    import unittest
    # use the zope.testrunner machinery to find all the
    # test suites we've put under ourselves
    import zope.testrunner.find
    import zope.testrunner.options
    here = os.path.abspath(os.path.join(os.path.dirname(__file__), 'src'))
    args = sys.argv[:]
    defaults = ["--test-path", here]
    options = zope.testrunner.options.get_options(args, defaults)
    suites = list(zope.testrunner.find.find_suites(options))
    return unittest.TestSuite(suites)


TESTS_REQUIRE = [
    'zope.browsermenu',
    'zope.testing',
    'zope.testrunner',
]

setup(
    name='zope.browserpage',
    version='4.4.0',
    url='https://github.com/zopefoundation/zope.browserpage',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.org',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope :: 3',
    ],
    description='ZCML directives for configuring browser views for Zope.',
    long_description=long_description,
    license='ZPL 2.1',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['zope'],
    install_requires=[
        'setuptools',
        'zope.tal >= 4.2.0',
        'zope.pagetemplate',
        'zope.component >= 3.7',
        'zope.configuration',
        'zope.interface',
        'zope.publisher >= 3.8',
        'zope.schema',
        'zope.security',
        'zope.traversing',
        ],
    extras_require={
        'menu': [
            'zope.browsermenu',
        ],
        'test': TESTS_REQUIRE,
    },
    tests_require=TESTS_REQUIRE,
    test_suite='__main__.alltests',
    include_package_data=True,
    zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2005-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
"""

from zope import component, interface
import zope.traversing.interfaces

class INamedTemplate(interface.Interface):
    """A template that is looked up by name
    """

class NamedTemplateImplementation(object):

    def __init__(self, descriptor, view_type=None):
        try:
            descriptor.__get__
        except AttributeError:
            raise TypeError(
                "NamedTemplateImplementation must be passed a descriptor."
                )
        self.descriptor = descriptor
        interface.implementer(INamedTemplate)(self)

        if view_type is not None:
            component.adapter(view_type)(self)

    def __call__(self, instance):
        return self.descriptor.__get__(instance, instance.__class__)


class implementation(object):

    def __init__(self, view_type=None):
        self.view_type = view_type

    def __call__(self, descriptor):
        return NamedTemplateImplementation(descriptor, self.view_type)


class NamedTemplate(object):

    def __init__(self, name):
        self.__name__ = name

    def __get__(self, instance, type=None):
        if instance is None:
            return self
        return component.getAdapter(instance, INamedTemplate, self.__name__)

    def __call__(self, instance, *args, **kw):
        self.__get__(instance)(*args, **kw)


@interface.implementer(zope.traversing.interfaces.IPathAdapter)
class NamedTemplatePathAdapter(object):

    def __init__(self, context):
        self.context = context

    def __getitem__(self, name):
        return component.getAdapter(self.context, INamedTemplate, name)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""File-based page templates that can be used as methods on views.
"""
__docformat__ = 'restructuredtext'

from zope.component import getMultiAdapter
from zope.pagetemplate.pagetemplatefile import PageTemplateFile
from zope.pagetemplate.engine import TrustedAppPT

class ViewPageTemplateFile(TrustedAppPT, PageTemplateFile):
    """Page Templates used as methods of views defined as Python classes.
    """

    def __init__(self, filename, _prefix=None, content_type=None):
        _prefix = self.get_path_from_prefix(_prefix)
        super(ViewPageTemplateFile, self).__init__(filename, _prefix)
        if content_type is not None:
            self.content_type = content_type

    def pt_getContext(self, instance, request, **_kw):
        # instance is a View component
        namespace = super(ViewPageTemplateFile, self).pt_getContext(**_kw)
        namespace['request'] = request
        namespace['view'] = instance
        namespace['context'] = context = instance.context
        namespace['views'] = ViewMapper(context, request)
        return namespace

    def __call__(self, instance, *args, **keywords):
        namespace = self.pt_getContext(
            request=instance.request,
            instance=instance, args=args, options=keywords)
        debug_flags = instance.request.debug
        s = self.pt_render(
            namespace,
            showtal=getattr(debug_flags, 'showTAL', 0),
            sourceAnnotations=getattr(debug_flags, 'sourceAnnotations', 0),
            )
        response = instance.request.response
        if not response.getHeader("Content-Type"):
            response.setHeader("Content-Type", self.content_type)
        return s

    def __get__(self, instance, type):
        return BoundPageTemplate(self, instance)

class ViewMapper(object):
    def __init__(self, ob, request):
        self.ob = ob
        self.request = request

    def __getitem__(self, name):
        return getMultiAdapter((self.ob, self.request), name=name)


class BoundPageTemplate(object):
    def __init__(self, pt, ob):
        object.__setattr__(self, '__func__', pt)
        object.__setattr__(self, '__self__', ob)

    macros = property(lambda self: self.__func__.macros)
    filename = property(lambda self: self.__func__.filename)

    def __call__(self, *args, **kw):
        if self.__self__ is None:
            im_self, args = args[0], args[1:]
        else:
            im_self = self.__self__
        return self.__func__(im_self, *args, **kw)

    def __setattr__(self, name, v):
        raise AttributeError("Can't set attribute", name)

    def __repr__(self):
        return "<BoundPageTemplateFile of %r>" % self.__self__

def NoTraverser(ob, request):
    return None
